/*
Copyright (C) 2013 Bengt Martensson.

This program is free software: you can redistribute it and/or modify
it under the terms of the GNU General Public License as published by
the Free Software Foundation; either version 3 of the License, or (at
your option) any later version.

This program is distributed in the hope that it will be useful, but
WITHOUT ANY WARRANTY; without even the implied warranty of
MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE. See the GNU
General Public License for more details.

You should have received a copy of the GNU General Public License along with
this program. If not, see http://www.gnu.org/licenses/.
*/

package org.harctoolbox.irscrutinizer.importer;

import java.net.URL;
import java.util.ArrayList;
import java.util.Collection;
import java.util.HashMap;
import java.util.LinkedHashMap;
import org.harctoolbox.IrpMaster.IrpMasterException;
import org.harctoolbox.IrpMaster.IrpUtils;
import org.harctoolbox.IrpMaster.ModulatedIrSequence;
import org.harctoolbox.girr.Command;

/**
 * This class implements functionality common to importers. It does not know to read stuff.
 */
public abstract class Importer {
    // This can contain multiple commands with the same name
    private final ArrayList<Command> commands;

    // This has, of course, unique names
    private final LinkedHashMap<String,Command>commandIndex;

    // origin is set by load, not the constructor.
    protected String origin = null;
    protected String creatingUser = System.getProperty("user.name", "unknown");
    protected boolean invokeDecodeIr = false;
    protected boolean generateRaw = false;
    protected boolean generateCcf = false;
    protected double fallbackFrequency = IrpUtils.defaultFrequency;

    /**
     * @param generateRaw the generateRaw to set
     */
    public void setGenerateRaw(boolean generateRaw) {
        this.generateRaw = generateRaw;
    }

    /**
     * @param generateCcf the generateCcf to set
     */
    public void setGenerateCcf(boolean generateCcf) {
        this.generateCcf = generateCcf;
    }

    /**
     * @param invokeDecodeIr the invokeDecodeIr to set
     */
    public void setInvokeDecodeIr(boolean invokeDecodeIr) {
        this.invokeDecodeIr = invokeDecodeIr;
    }

    /**
     * @param creatingUser the creatingUser to set
     */
    public void setCreatingUser(String creatingUser) {
        this.creatingUser = creatingUser;
    }

    /**
     * @param fallbackFrequency the fallbackFrequency to set
     */
    public void setFallbackFrequency(double fallbackFrequency) {
        this.fallbackFrequency = fallbackFrequency;
    }

    protected void prepareLoad(String origin) {
        this.origin = origin;
        clearCommands();
    }

    protected Importer() {
        commands = new ArrayList<Command>();
        commandIndex = new LinkedHashMap<String,Command>();
    }

    protected String uniqueName(String name) {
        String uniqueName = name;
        int index = 1;
        while (commandIndex.containsKey(uniqueName))
            uniqueName = name + '$' + Integer.toString(index++);

        return uniqueName;
    }

    protected boolean addCommand(Command command) {
        commands.add(command);
        if (commandIndex.containsKey(command.getName()))
            return false;

        commandIndex.put(command.getName(), command);
        return true;
    }

    protected boolean addCommands(Collection<Command> commands) {
        boolean totalStatus = true;
        for (Command command : commands) {
            boolean status = addCommand(command);
            totalStatus = totalStatus && status;
        }
        return totalStatus;
    }

    public ArrayList<Command> getCommands() {
        return commands;
    }

    public Command getCommand(String name) {
        return commandIndex.get(name);
    }

    public ModulatedIrSequence getConcatenatedCommands() throws IrpMasterException {
        ModulatedIrSequence[] array = new ModulatedIrSequence[commands.size()];
        int index = 0;
        for (Command command : commands) {
            array[index++] = command.toModulatedIrSequence(1);
        }
        return new ModulatedIrSequence(array);
    }

    public HashMap<String,Command> getCommandIndex() {
        return commandIndex;
    }

    protected void clearCommands() {
        commands.clear();
        commandIndex.clear();
    }

    public String getOrigin() {
        return origin;
    }

    public URL getHomeUrl() {
        return null;
    }
}
